@@ -168,11 +168,11 @@ class History:
         """Flush the history items to disk from a buffer."""
         pass
 
-    def items(self):
+    def items(self, newest_first=False):
         """Get history items of current session."""
         raise NotImplementedError
 
-    def all_items(self):
+    def all_items(self, newest_first=False):
         """Get all history items."""
         raise NotImplementedError
 
@@ -210,11 +210,11 @@ class DummyHistory(History):
     def append(self, cmd):
         pass
 
-    def items(self):
+    def items(self, newest_first=False):
         yield {'inp': 'dummy in action', 'ts': 1464652800, 'ind': 0}
 
-    def all_items(self):
-        return self.items()
+    def all_items(self, newest_first=False):
+        return self.items(newest_first=newest_first)
 
     def info(self):
         data = collections.OrderedDict()
@@ -288,9 +288,9 @@ def _xhj_gc_bytes_to_rmfiles(hsize, files):
     return rmfiles
 
 
-def _xhj_get_history_files(sort=True, reverse=False):
+def _xhj_get_history_files(sort=True, newest_first=False):
     """Find and return the history files. Optionally sort files by
-        modify time.
+    modify time.
     """
     data_dir = builtins.__xonsh_env__.get('XONSH_DATA_DIR')
     data_dir = xt.expanduser_abs_path(data_dir)
@@ -302,7 +302,7 @@ def _xhj_get_history_files(sort=True, reverse=False):
         if builtins.__xonsh_env__.get('XONSH_DEBUG'):
             xt.print_exception("Could not collect xonsh history files.")
     if sort:
-        files.sort(key=lambda x: os.path.getmtime(x), reverse=reverse)
+        files.sort(key=lambda x: os.path.getmtime(x), reverse=newest_first)
     return files
 
 
@@ -603,12 +603,16 @@ class JsonHistory(History):
         self.buffer.clear()
         return hf
 
-    def items(self):
+    def items(self, newest_first=False):
         """Display history items of current session."""
-        for item, tss in zip(self.inps, self.tss):
+        if newest_first:
+            items = zip(reversed(self.inps), reversed(self.tss))
+        else:
+            items = zip(self.inps, self.tss)
+        for item, tss in items:
             yield {'inp': item.rstrip(), 'ts': tss[0]}
 
-    def all_items(self, **kwargs):
+    def all_items(self, newest_first=False, **kwargs):
         """
         Returns all history as found in XONSH_DATA_DIR.
 
@@ -616,7 +620,7 @@ class JsonHistory(History):
         """
         while self.gc and self.gc.is_alive():
             time.sleep(0.011)  # gc sleeps for 0.01 secs, sleep a beat longer
-        for f in _xhj_get_history_files():
+        for f in _xhj_get_history_files(newest_first=newest_first):
             try:
                 json_file = xlj.LazyJSON(f, reopen=False)
             except ValueError:
@@ -630,6 +634,8 @@ class JsonHistory(History):
                     msg = 'xonsh history file {0!r} is not valid JSON'
                     print(msg.format(f), file=sys.stderr)
                 continue
+            if newest_first:
+                commands = reversed(commands)
             for c in commands:
                 yield {'inp': c['inp'].rstrip(), 'ts': c['ts'][0]}
         # all items should also include session items
@@ -733,14 +739,14 @@ def _xh_sqlite_get_count(cursor, sessionid=None):
     return cursor.fetchone()[0]
 
 
-def _xh_sqlite_get_records(cursor, sessionid=None, limit=None, reverse=False):
+def _xh_sqlite_get_records(cursor, sessionid=None, limit=None, newest_first=False):
     sql = 'SELECT inp, tsb, rtn FROM xonsh_history '
     params = []
     if sessionid is not None:
         sql += 'WHERE sessionid = ? '
         params.append(sessionid)
     sql += 'ORDER BY tsb '
-    if reverse:
+    if newest_first:
         sql += 'DESC '
     if limit is not None:
         sql += 'LIMIT %d ' % limit
@@ -776,11 +782,11 @@ def xh_sqlite_get_count(sessionid=None, filename=None):
         return _xh_sqlite_get_count(c, sessionid=sessionid)
 
 
-def xh_sqlite_items(sessionid=None, filename=None):
+def xh_sqlite_items(sessionid=None, filename=None, newest_first=False):
     with _xh_sqlite_get_conn(filename=filename) as conn:
         c = conn.cursor()
         _xh_sqlite_create_history_table(c)
-        return _xh_sqlite_get_records(c, sessionid=sessionid)
+        return _xh_sqlite_get_records(c, sessionid=sessionid, newest_first=newest_first)
 
 
 def xh_sqlite_delete_items(size_to_keep, filename=None):
@@ -863,15 +869,15 @@ class SqliteHistory(History):
             cmd, str(self.sessionid), store_stdout,
             filename=self.filename)
 
-    def all_items(self):
+    def all_items(self, newest_first=False):
         """Display all history items."""
-        for item in xh_sqlite_items(filename=self.filename):
+        for item in xh_sqlite_items(filename=self.filename, newest_first=newest_first):
             yield {'inp': item[0], 'ts': item[1], 'rtn': item[2]}
 
-    def items(self):
+    def items(self, newest_first=False):
         """Display history items of current session."""
         for item in xh_sqlite_items(
-                sessionid=str(self.sessionid), filename=self.filename):
+                sessionid=str(self.sessionid), filename=self.filename, newest_first=newest_first):
             yield {'inp': item[0], 'ts': item[1], 'rtn': item[2]}
 
     def info(self):
@@ -935,18 +941,18 @@ def construct_history(**kwargs):
     return kls_history(**kwargs)
 
 
-def _xh_session_parser(hist=None, **kwargs):
+def _xh_session_parser(hist=None, newest_first=False, **kwargs):
     """Returns history items of current session."""
     if hist is None:
         hist = builtins.__xonsh_history__
-    return hist.items()
+    return hist.items(newest_first=newest_first)
 
 
-def _xh_all_parser(hist=None, **kwargs):
+def _xh_all_parser(hist=None, newest_first=False, **kwargs):
     """Returns all history items."""
     if hist is None:
         hist = builtins.__xonsh_history__
-    return hist.all_items()
+    return hist.all_items(newest_first=newest_first)
 
 
 def _xh_find_histfile_var(file_list, default=None):
